//
//This sample program is provided AS IS and may be used, executed, copied and
//modified without royalty payment by customer (a) for its own instruction and 
//study, (b) in order to develop applications designed to run with an IBM 
//product, either for customer's own internal use or for redistribution 
//by customer, as part of such an application, in customer's own products. "
//
//5724-J34 (C) COPYRIGHT International Business Machines Corp. 2015
//All Rights Reserved * Licensed Materials - Property of IBM
//
var WXS = require('./wxs');
var debug = require('debug')('cache');
var cfenv = require('cfenv');
var wxsclient;

// operation instructions for quick memory store in case the cloud foundry environment is not configured
var stub = {
    cache: new Object(),

    put: function(key, val, cb) {
        this.cache[key] = val;
        cb(null,JSON.stringify(val));
    },

    get: function(key, cb) {
        cb(null,this.cache[key]);
    },

    remove: function(key, cb) {
        delete this.cache[key];
        cb();
    },
    
    updateOptions: function(opts) {
    	
    }
};

if(isCfConfigured()) {
	var appEnv = cfenv.getAppEnv();
	var wxsprops = getEnv(appEnv.services);
	wxsclient = new WXS(wxsprops);
} 
else {
    debug("WARNING!: using non-persistent in memory storage (NOT default express-session) instead of DataCache.");
    wxsclient = stub; 
}

exports.getCache = function(key,callback) {
	wxsclient.get(key, callback);
};

exports.putCache = function(key, value,callback) {
	wxsclient.put(key, value, callback);
};

exports.removeCache = function(key,callback) {
	var key = req.params.key;
	wxsclient.remove(key, function() {
		res.json({
			value : "The delete was successful for the key " + key + "."
		});
		debug('finished remove');
	});
};

exports.update = function(options) {
	debug("cache updating options for grid name " + options);
	wxsclient.updateOptions(options);
};
	
exports.isCfEnvironment = function() {
	return isCfConfigured();
};

//verify if we are in a Cloud-Foundry OK environment-whether that is live on CF or local with programmer
//defined VCAP overrides
function isCfConfigured() {
	var applEnv = cfenv.getAppEnv();
	if(applEnv.services) {
		// verify that the DataCache section of the provided VCAP-where ever it is from-is well formed
		var serviceprops = getEnv(applEnv.services);
		if(typeof serviceprops != "undefined" && serviceprops.hasOwnProperty("gridName")) {
			  // looks good, service definition for Data Cache is there
			  return true;
		}
		else {
			// service object is defined, but not well formed for Data Cache service
			return false;
		}
	}
	else {
		// no CF environment defined, return false
		return false;
	}
}

/**
 * Need to ignore the version number of DataCache when getting the credentials.
 */
function getEnv(vcapEnv) {
   for (var serviceOfferingName in vcapEnv) {
   	    if (vcapEnv.hasOwnProperty(serviceOfferingName) &&
   	    		serviceOfferingName.indexOf("DataCache-") === 0) {
   	    	var serviceBindingData = vcapEnv[serviceOfferingName][0];
   	    	return serviceBindingData.credentials;
   	    }
   }
}