//
//This sample program is provided AS IS and may be used, executed, copied and
//modified without royalty payment by customer (a) for its own instruction and 
//study, (b) in order to develop applications designed to run with an IBM 
//product, either for customer's own internal use or for redistribution 
//by customer, as part of such an application, in customer's own products. "
//
//5724-J34 (C) COPYRIGHT International Business Machines Corp. 2015
//All Rights Reserved * Licensed Materials - Property of IBM
//
var url = require('url');
var debug = require('debug')('llcache');
var http = require('http');
var querystring = require("querystring");

var cookie ='';
var ttl = null;

function setCookieIfNeeded(res) {
	if (null != res.headers['set-cookie']) {
	   if (Array.isArray(res.headers['set-cookie'])) {
		   	  cookie = '';
			  res.headers['set-cookie'].forEach(function(cook) {
				 cookie =cook.split(';')[0]+'; '+cookie;
			  });
		} else {
				cookie = res.headers['set-cookie'].split(';')[0];
			}
	}
}

function WXS(properties) {
	this.wxs = properties;
	if (this.wxs.restResource.indexOf('http://') !== 0) {
		this.wxs.restResource = 'http://' + this.wxs.restResource;
	}
	this.auth = 'Basic ' + new Buffer(this.wxs.username + ':' + this.wxs.password).toString('base64');
	this.parsed = url.parse(this.wxs.restResource);
	debug("wxsclient url=" + this.wxs.restResource);
}

WXS.prototype = {
		
	put : function(key, value, callback) {
		var uripath =  this.parsed.pathname + '/' + this.wxs.gridName + '/' + encodeURIComponent(key);
		if (this.ttl){
			uripath = uripath +  "?ttl=" + this.ttl;
		}
		var post_options = {
			hostname : this.parsed.hostname,
			port : '80',
			path : uripath,
			method : 'POST',
			headers : {
				'Content-Type' : 'application/json',
				'Connection': 'keep-alive',
				'Cookie': cookie,
				'Authorization' : this.auth
			},
			rejectUnauthorized : false,
			agent : false,
		};
		var post_req = http.request(post_options, function(res) {
			res.setEncoding('utf8');
			res.on('data', function(chunk) {
			});
			res.on('error', function(c) {
				debug('post error: ' + c);
			});
			res.on('end', function() {
				debug('post status ' + res.statusCode);
				if (res.statusCode == 200) {
					setCookieIfNeeded(res);
					callback(null);
				}
				else {
					callback("Error placing session into the cache");
				}
			});
		});
		debug('post_req is'+post_options.path);
		post_req.write(JSON.stringify(value));
		post_req.end();
	},
	get : function(key, callback) {
		var get_options = {
			hostname : this.parsed.hostname,
			port : '80',
			path : this.parsed.pathname + '/' + this.wxs.gridName + '/' + encodeURIComponent(key),
			method : 'GET',
			headers : {
				'Content-Type' : 'application/json',
				'Connection': 'keep-alive',
				'Cookie': cookie,
				'Authorization' : this.auth
			},
			rejectUnauthorized : false,
			agent : false,
		};
		var get_req = http.request(get_options, function(res) {
			var resultString = '';
			res.on('data', function(chunk) {
				debug('get response: ' + chunk);
				resultString += chunk;
			});
			res.on('error', function(c) {
				debug('get error: ' + c);
			});
			res.on('end', function() {
				debug('get status ' + res.statusCode);
				setCookieIfNeeded(res);
				if (res.statusCode == 200 ) {
					callback(null,JSON.parse(resultString));
				} 
				else if ( res.statusCode == 404) {
					// key not found, but no error
					callback(null,null);
				}
				else {
					var err = "return code code from REST gateway is "+ res.statusCode;
					callback(err,null);
				}
			});
		});
		get_req.end();
	},
	remove : function(key, callback) {
		var get_options = {
			hostname : this.parsed.hostname,
			path : this.parsed.pathname + '/' + this.wxs.gridName + '/' +
					encodeURIComponent(key),
			port : '80',
			method : 'DELETE',
			headers : {
				'Content-Type' : 'application/json',
				'Connection': 'keep-alive',
				'Cookie': cookie,
				'Authorization' : this.auth
			},
			rejectUnauthorized : false,
			agent : false,
		};
		var get_req = http.request(get_options, function(res) {
			var resultString = '';
			res.on('data', function(chunk) {
				debug('get response: ' + chunk);
				resultString += chunk;
			});
			res.on('error', function(c) {
				debug('get error: ' + c);
			});
			res.on('end', function() {
				debug('delete status ' + res.statusCode);
				if (res.statusCode == 200) {
					setCookieIfNeeded(res);
					callback(null);
				}
				else {
					callback("error removing session from cache");
				}
			});
		});
		get_req.end();
	},
	updateOptions : function(options) {
		if (options.mapname != null) {
			debug("wxs updating grid name"+options.mapname);
			this.wxs.gridName = options.mapname;
		}
		
		if (options.eviction != null) {
			debug("wxs updating grid eviction"+options.eviction);
			this.wxs.gridName += '.' + options.eviction;
		} else {
			this.wxs.gridName += '.LUT';
		}
		
		if (options.ttl != null)
		{
			this.ttl = options.ttl;
		}
	}
};

module.exports = WXS;
